@@ -13,6 +13,7 @@ use JSON::PP;
 use File::Copy;
 use File::Basename;
 use MIME::Base64;
+use TeX::Hyphen;
 
 use Getopt::Long;
 
@@ -21,11 +22,14 @@ my $Out     = '';
 my $Version = '1.0';
 my $Lang    = 'ru';
 my $ArtID   = undef;
+my $Dictionary = undef;
+my $Hyp = undef;
 
 GetOptions ('in|from|src|fb3=s' => \$FB3,
             'out|to|dst|json=s' => \$Out,
             'lang=s'            => \$Lang,
             'art|art-id=s'      => \$ArtID,
+            'dict=s'            => \$Dictionary,
             'version=s'         => \$Version) or print join('', <DATA>) and die("Error in command line arguments\n");
 
 print join('', <DATA>) and die "ERROR: source directory not specified, use --fb3 parameter\n"       unless $FB3;
@@ -40,6 +44,18 @@ unless ($Version =~ /^\d+\.\d+$/) {
 	$Version = ($Version =~ /^\d+$/) ? "1.$Version" : "1.0"
 }
 
+my $CannotHyph;
+if ($Dictionary) {
+	if (-e $Dictionary) {
+		$Hyp = new TeX::Hyphen 'file' => $Dictionary,
+			'style' => 'utf8', leftmin => 2, rightmin => 2;
+	} else {
+		die "\nERROR: dictionary file `$Dictionary' not found\n"
+	}
+} elsif ($Lang eq 'pl') {
+	$CannotHyph = 1; 
+}
+
 my $PartLimit = 20000;
 my $IsTrial   = 0;
 
@@ -148,8 +164,8 @@ $hyphenPatterns = {
 $hyphenRegexPattern = join "|",keys %{$hyphenPatterns};
 $hyphenRegexPattern = qr/(.*)($hyphenRegexPattern){1}(.*)/o;
 
-$soglasnie = "bcdfghjklmnpqrstvwxzбвгджзйклмнпрстфхцчшщ";
-$glasnie = "aeiouyАОУЮИЫЕЭЯЁєіїў";
+$soglasnie = "bcdfghjklmnpqrstvwxzбвгджзйклмнпрстфхцчшщłćżźśńż";
+$glasnie = "aeiouyАОУЮИЫЕЭЯЁєіїўóąę";
 $znaki = "ъь";
 
 $RgxSoglasnie = qr/[$soglasnie]/oi;
@@ -161,8 +177,12 @@ $RgxNonChar = qr/([^$soglasnie$glasnie$znaki]+)/oi; #в скобках, чтоб
 my $jsonC = JSON::PP->new->pretty->allow_barekey;
 
 my $FB3Package = FB3->new( from_dir => $FB3 );
-my $TOCHeader = ProceedDescr($FB3Package->Meta->Content);
 my $FB3Body = $FB3Package->Body;
+
+my $Parser = XML::LibXML->new();
+my $BodyDoc = $Parser->load_xml( string => $FB3Body->Content, huge => 1 );
+
+my $TOCHeader = ProceedDescr($FB3Package->Meta->Content);
 my @Img = $FB3Body->Relations( type => RELATION_TYPE_FB3_IMAGES );
 my @ImgFiles;
 for (@Img) {
@@ -196,8 +216,6 @@ sub ProceedBody {
 	$BodyXML = DecodeUtf8($BodyXML);
 	$BodyXML =~ s/\r?\n\r?/ /g;
 	$BodyXML =~ s/([\s>])([^\s<>]+)(<note\s+[^>]*?role="(foot|end)note"[^>]*?>[^<]{1,10}<\/note>[,\.\?"'“”«»‘’;:\)…\/]?)/$1.HypheNOBR($2,$3)/ges;
-	my $Parser = XML::LibXML->new();
-	my $BodyDoc = $Parser->load_xml( string => $BodyXML, huge => 1 );
 	my $JsonStr;
 	my $TOCStr;
 	my $TotalBlocks;
@@ -676,13 +694,21 @@ sub ProceedDescr {
 		$description .= $DraftStr;
 	}
 
-	my $FragmentStr = '';
+	my $LengthStr = '';
 	if ( my $FragmentNode = $xpc->findnodes('/fbd:fb3-description/fbd:fb3-fragment')->[0] ) {
 		$IsTrial = 1;
-		$FragmentStr = ",\n".'"fb3-fragment":{"full_length":'.$FragmentNode->getAttribute('full_length').',"fragment_length":'.$FragmentNode->getAttribute('fragment_length').'}';
+		$LengthStr = ",\n".'"fb3-fragment":{"full_length":'.$FragmentNode->getAttribute('full_length').',"fragment_length":'.$FragmentNode->getAttribute('fragment_length').'}';
+	} else {
+		my $RootNode = $BodyDoc->getDocumentElement();
+		my $CharsFull = length($RootNode->textContent);
+		$xpc->registerNs('fbb', &NS_FB3_BODY);
+		foreach my $TrialOnlyNode ($xpc->findnodes('/fbb:fb3-body/fbb:section[@output="trial-only"]',$RootNode)) {
+			$CharsFull -= (length($TrialOnlyNode->textContent) || 0);
+		}
+		$LengthStr = ",\n".'"full_length":'.$CharsFull;
 	}
-	
-	return 'Meta:{' . $description . ',UUID:"' . $UUID . '",version:"' . $Version . '"}' . $FragmentStr . ",\n";
+
+	return 'Meta:{' . $description . ',UUID:"' . $UUID . '",version:"' . $Version . '"}' . $LengthStr . ",\n";
 }
 
 sub DecodeUtf8 {
@@ -788,12 +814,18 @@ sub trim {
 sub HyphString {
 	use utf8;
 	my $word = shift;
-	return $word if $Lang eq 'pl';
+	return $word if $CannotHyph;
 	my @wordArrayWithUnknownSymbols = split $RgxNonChar , $word; #собрали все слова и неизвестные символы. Для слова "пример!№?;слова" будет содержать "пример", "!№?;", "слова".
 
 	for my $word (@wordArrayWithUnknownSymbols) {
 		next if $word =~ $RgxNonChar;
-		$word = HyphParticularWord($word);
+		if (-e $Dictionary) {
+			$word = $Hyp->visualize($word);
+			$word =~ s/-/\x{AD}/g;
+		} else {
+			next if $Lang eq 'pl';
+			$word = HyphParticularWord($word);
+		}
 	}
 	return join "", @wordArrayWithUnknownSymbols;
 }
@@ -832,7 +864,7 @@ __DATA__
 
 Usage:
 
-    fb3_2_json.pl --fb3 /path/to/fb3/dir --json /path/to/json/dir [ --version <file version> ] [ --lang <file language> ] [ --art-id <id> ]
+    fb3_2_json.pl --fb3 /path/to/fb3/dir --json /path/to/json/dir [ --version <file version> ] [ --lang <file language> ] [ --art-id <id> ] [ --dict <path> ]
 
 e.g.
 
@@ -841,5 +873,3 @@ e.g.
     fb3_2_json.pl --fb3 /tmp/fb3 --json /tmp/json --lang ru --art-id 1234567
 
     fb3_2_json.pl --fb3 /tmp/fb3 --json /tmp/json --version 2.1 --lang es --art-id 1234567
-
-
